@@ -274,14 +274,23 @@
 ;;; Otherwise, evaluate DEFAULT, store the resulting value in
 ;;; HASH-TABLE and return two values: 1) the result of evaluating
 ;;; DEFAULT 2) NIL.
-(defmacro ensure-gethash (key hash-table &optional default)
+;;; If ATOMICP is true, perform the lookup and potential update
+;;; atomically.
+(defmacro ensure-gethash (key hash-table &optional default atomicp)
+  (check-type atomicp boolean)
   (with-unique-names (n-key n-hash-table value foundp)
-    `(let ((,n-key ,key)
-           (,n-hash-table ,hash-table))
-       (multiple-value-bind (,value ,foundp) (gethash ,n-key ,n-hash-table)
-         (if ,foundp
-             (values ,value t)
-             (values (setf (gethash ,n-key ,n-hash-table) ,default) nil))))))
+    (flet ((probe-and-update (&optional update)
+             `(multiple-value-bind (,value ,foundp) (gethash ,n-key ,n-hash-table)
+                (if ,foundp
+                    (values ,value t)
+                    ,(or update
+                         `(values (setf (gethash ,n-key ,n-hash-table) ,default) nil))))))
+      `(let ((,n-key ,key)
+             (,n-hash-table ,hash-table))
+         ,(if atomicp
+              (probe-and-update `(with-locked-system-table (,n-hash-table)
+                                   ,(probe-and-update)))
+              (probe-and-update))))))
 
 ;; This is not an 'extension', but is needed super early, so ....
 (defmacro sb-xc:defconstant (name value &optional (doc nil docp))
