@@ -115,7 +115,7 @@
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
-    (inst .skip (* (1- simple-fun-code-offset) n-word-bytes))
+    (inst .skip (* (1- simple-fun-insts-offset) n-word-bytes))
     (let ((entry-point (gen-label)))
       (emit-label entry-point)
       ;; FIXME alpha port has a ### note here saying we should "save it
@@ -742,7 +742,7 @@ default-value-8
                     (inst beq step-done-label)
                     ;; CONTEXT-PC will be pointing here when the
                     ;; interrupt is handled, not after the UNIMP.
-                    (note-this-location vop :step-before-vop)
+                    (note-this-location vop :internal-error)
                     ;; Construct a trap code with the low bits from
                     ;; SINGLE-STEP-AROUND-TRAP and the high bits from
                     ;; the register number of CALLABLE-TN.
@@ -796,7 +796,7 @@ default-value-8
                   ;; is calculated.
                   (insert-step-instrumenting function)
                   (inst addi entry-point function
-                        (- (ash simple-fun-code-offset word-shift)
+                        (- (ash simple-fun-insts-offset word-shift)
                            fun-pointer-lowtag))))
                (:direct
                 `((inst lwz entry-point null-tn (static-fun-offset fun)))))
@@ -1119,6 +1119,20 @@ default-value-8
   (:variant 0 0)
   (:translate %more-arg))
 
+(define-vop (more-arg-or-nil)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg) :to (:result 1))
+         (count :scs (any-reg)))
+  (:info index)
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 3
+    (inst cmpwi count (fixnumize index))
+    (move value null-tn)
+    (inst ble done)
+    (inst lwz value object (ash index word-shift))
+    done))
+
 ;;; Turn more arg (context, count) into a list.
 (define-vop (listify-rest-args)
   (:args (context-arg :target context :scs (descriptor-reg))
@@ -1242,7 +1256,7 @@ default-value-8
     (inst beq DONE)
     ;; CONTEXT-PC will be pointing here when the interrupt is handled,
     ;; not after the UNIMP.
-    (note-this-location vop :step-before-vop)
+    (note-this-location vop :internal-error)
     ;; CALLEE-REGISTER-OFFSET isn't needed for before-traps, so we
     ;; can just use a bare SINGLE-STEP-BEFORE-TRAP as the code.
     (inst unimp single-step-before-trap)
